/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sample.tswebclient.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.faces.event.ActionEvent;
import javax.faces.event.AjaxBehaviorEvent;
import javax.inject.Inject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sample.tswebclient.cassandra.Application;
import sample.tswebclient.cassandra.CassandraManagerEJB;
import sample.tswebclient.cassandra.Attributes;
import sample.tswebclient.cassandra.ChildrenReadOption;
import sun.misc.BASE64Encoder;


/**
 * <p>The only JSF Managed bean that lists the terminal services applications. The
 * bean is injected using CDI named annotation, and the EJB is also injected
 * using CDI Inject annotation.</p>
 *
 * <p>The main idea of the bean is very simple: The list of applications is
 * always created from root folder "/" and a summary on the right is
 * shown when the cursor passes over any folder or application. The "tree"
 * list is used to print the applications tree and the "map" is a hashmap
 * to quickly find an application by its key (when the app is clicked
 * for example). Only one hierarchy of folders can be opened at the same time
 * (when the user clicks in another folder the previous one is closed). When
 * an application is clicked the FileDownloadServlet is invoked. For more
 * information see the index.xhtml facelets page.</p>
 *
 * <p>But two more ideas are important:
 * <ul>
 * <li>clickedFolder. The clicked folder key is stored to know which folders
 *     have been opened by the user and their children must be displayed. This
 *     variable controls the tree.</li>
 * <li>selectedApplication. The selected application var just controls the last
 *     application the cursor has passed over. This way the informational div
 *     shows the information of this app.</li>
 * </ul>
 *
 * @see FileDownloadServlet
 * @see ApplicationItem
 * @author ricky
 */
@javax.inject.Named("applicationTreeBean")
@javax.enterprise.context.SessionScoped
public class ApplicationTreeBean implements Serializable {

    private static final Logger logger = LoggerFactory.getLogger(ApplicationTreeBean.class);

    @Inject
    private CassandraManagerEJB manager;
    
    private String clickedFolder = null;
    private List<ApplicationItem> tree = null;
    private Map<String,Application> map = null;
    private Application selectedApplication = null;

    /**
     * Empty constructor.
     */
    public ApplicationTreeBean() {
        logger.info("creating...");
        tree = new ArrayList<ApplicationItem>();
        map = new HashMap<String,Application>();
        clickedFolder = null;
    }

    //
    // GETTERS & SETTERS
    //

    /**
     * Return the current application clicked (the key).
     * @return Key of the current application clicked.
     */
    public String getClickedFolder() {
        return clickedFolder;
    }

    /**
     * Changes the current clicked application.
     * @param clickedFolder The key of the new clicked application.
     */
    public void setClickedFolder(String clickedFolder) {
        this.clickedFolder = clickedFolder;
    }

    /**
     * Return the tree in order to print it by the xhtml.
     * @return The application item list.
     */
    public List<ApplicationItem> getTree() {
        logger.info("getTree: " + tree);
        if (tree.isEmpty()) {
            doList();
        }
        return tree;
    }

    /**
     * This method returns if the key passed is a open folder. This is just
     * to know if the clicked folder starts with the specified key (for example
     * if clicked folder is "/APPLICATIONS/INTERNET", "/APPLICATIONS" is an open
     * folder, but "/GAMES" isn't).
     *
     * @param key The application key to check.
     * @return True if the folder is opened, false if not.
     */
    public boolean isOpenFolder(String key) {
        return clickedFolder != null && 
                (clickedFolder.startsWith(key + "/") || clickedFolder.equals(key));
    }

    /**
     * This method return the icon but in "data:image/png;base64," format. This
     * way is very easy to send application icons using JSF.
     *
     * @param img The png image to encode.
     * @return The img in html format.
     */
    public String encodeImage(byte[] img) {
        StringBuilder sb = new StringBuilder();
        sb.append("data:image/png;base64,");
        BASE64Encoder encoder = new BASE64Encoder();
        sb.append(encoder.encode(img));
        return sb.toString();
    }

    /**
     * The selected application (the last application the cursor has passed
     * over) is returned.
     * @return The key of the selected application.
     */
    public Application getSelectedApplication() {
        return selectedApplication;
    }

    //
    // PRIVATE METHODS
    //

    /**
     * Private method that reconstructs the map and tree reading the key
     * passed with FULL children. This way the father is read and then its
     * children, if the key is a folder with children and it is opened
     * the method is called recursively in order to print full opened tree.
     *
     * @param key The key to add to map and tree variables.
     * @param depth The current depth.
     * @throws Exception Some error.
     */
    private void createList(String key, int depth) throws Exception {
        logger.info("createList " + key + depth);
        // read the app an their children
        Application app = manager.readApplication(key,
                ChildrenReadOption.FULL,
                Attributes.name, Attributes.type, Attributes.icon, Attributes.description);
        logger.info("app: " + app);
        // create the list
        for (Application child: app.getChildren()) {
            // add this child to the list
            tree.add(new ApplicationItem(child, depth));
            map.put(child.getKey(), child);
            // if it is selected => add its children too
            if (isOpenFolder(child.getKey())) {
                createList(child.getKey(), depth + 1);
            }
        }
    }

    //
    // ACTIONS
    //

    /**
     * JSF listener executed when the cursor passes over an application.
     * The listener fills the selectedApplication key in order to show the
     * informational div presented on the right.
     *
     * @param event The Ajax Behavior Event
     */
    public void mouseOverListener(AjaxBehaviorEvent event) {
        String key = (String) event.getComponent().getAttributes().get("mouseKey");
        logger.info("mouseOverListener: " + key);
        selectedApplication = map.get(key);
        logger.info("mouseOverListener: " + selectedApplication);
    }

    /**
     * JSF listener executed when the cursor goes out the application. This
     * listener cleans the selectedApplication var to hide the informational
     * div on the right.
     * 
     * @param event The Ajax Behavior Event
     */
    public void mouseOutListener(AjaxBehaviorEvent event) {
        selectedApplication = null;
    }

    /**
     * jSF listener executed when a folder is clicked. The map and tree
     * are recalculated again with the new clickedFolder (this var controls
     * how the tree is expanded).
     *
     * @param event The Action Event
     */
    public void listListener(ActionEvent event) {
        Object key = event.getComponent().getAttributes().get("clickedKey");
        if (key != null) {
            clickedFolder = key.toString();
        }
        logger.warn("ActionEvent doList " + clickedFolder);
        doList();
    }

    /**
     * Method that cleans and recalculates (calling private createList method)
     * the map and tree using clickedFolder to know which folders are opened.
     *
     * @return "list"
     */
    public String doList() {
        try {
            tree.clear();
            map.clear();
            createList(Application.root().getKey(), 0);
        } catch (Exception e) {
            logger.error("createList error", e);
            FacesUtils.errorMessage(null, e);
            tree.clear();
            map.clear();
        }
        return "list";
    }


}
